@@ -1,4 +1,3 @@
-/* $Id$ */
 /*-
  * Copyright (c) 2006-2007 Benedikt Meurer <benny@xfce.org>.
  *
@@ -741,7 +740,8 @@ exo_mount_hal_device_mount (ExoMountHalDevice *device,
         {
           option = NULL;
 
-          if (strcmp (device->fsoptions[m], "sync") == 0)
+          if (strcmp (device->fsoptions[m], "sync") == 0
+              && device->volume == NULL)
             {
               /* non-pollable drive... */
               option = g_strdup ("sync");
